# Generated by roxygen2: do not edit by hand

S3method(print,pcache)
S3method(print,pdiff)
S3method(print,pdiff_score)
S3method(print,pinfo)
S3method(print,prepo)
S3method(print,rbreak)
S3method(print,rstability)
export(pkg_cache)
export(pkg_diff)
export(pkg_info)
export(pkg_repo)
export(pkg_stability)
export(pkg_summary)
export(pkg_versions)
export(repo_breakages)
export(repo_stability)
import(common)
import(cranlogs)
import(crayon)
import(grDevices)
import(graphics)
import(rvest)
import(tools)
import(utils)
