NAME:
  Package Changes Analyzer (pkgdiff) - a tool for visualizing changes in Linux software packages (RPM, DEB, TAR.GZ, etc). The tool is intended for Linux maintainers who are interested in ensuring compatibility of old and new versions of packages.
  
  Sample report: http://abi-laboratory.pro/tracker/package_diff/libssh/0.6.5/0.7.0/report.html
  
  The tool is developed by Andrey Ponomarenko: http://abi-laboratory.pro/

INSTALL:
  sudo make install prefix=/usr

REQUIRES:
  Perl 5
  GNU Diff
  GNU Wdiff
  GNU Awk
  GNU Binutils

USAGE:
  pkgdiff PKG1 PKG2

EXAMPLE:
  pkgdiff libssh-0.6.5.tar.xz libssh-0.7.0.tar.xz

ADV. USAGE:
  For advanced usage, see output of --help option

SUGGESTS:
  ABI Compliance Checker 1.99.1 or newer: https://github.com/lvc/abi-compliance-checker/
  ABI Dumper 0.97 or newer: https://github.com/lvc/abi-dumper
